@@ -19,12 +19,22 @@ use League\CommonMark\Block\Parser\BlockParserInterface;
 use League\CommonMark\Block\Renderer\BlockRendererInterface;
 use League\CommonMark\Delimiter\Processor\DelimiterProcessorInterface;
 use League\CommonMark\Environment;
-use League\CommonMark\EnvironmentAwareInterface;
+use League\CommonMark\Event\AbstractEvent;
 use League\CommonMark\Extension\ExtensionInterface;
 use League\CommonMark\Inline\Parser\InlineParserInterface;
 use League\CommonMark\Inline\Renderer\InlineRendererInterface;
+use League\CommonMark\Tests\Unit\Environment\FakeBlock1;
+use League\CommonMark\Tests\Unit\Environment\FakeBlock3;
+use League\CommonMark\Tests\Unit\Environment\FakeBlockParser;
+use League\CommonMark\Tests\Unit\Environment\FakeBlockRenderer;
+use League\CommonMark\Tests\Unit\Environment\FakeDelimiterProcessor;
+use League\CommonMark\Tests\Unit\Environment\FakeInline1;
+use League\CommonMark\Tests\Unit\Environment\FakeInline3;
+use League\CommonMark\Tests\Unit\Environment\FakeInlineParser;
+use League\CommonMark\Tests\Unit\Environment\FakeInlineRenderer;
 use League\CommonMark\Tests\Unit\Event\FakeEvent;
-use League\CommonMark\Util\ConfigurationAwareInterface;
+use League\CommonMark\Tests\Unit\Event\FakeEventListener;
+use League\CommonMark\Tests\Unit\Event\FakeEventListenerInvokable;
 use PHPUnit\Framework\TestCase;
 
 class EnvironmentTest extends TestCase
@@ -110,7 +120,7 @@ class EnvironmentTest extends TestCase
 
     public function testSetConfigAfterInit()
     {
-        $this->expectException('RuntimeException');
+        $this->expectException(\RuntimeException::class);
 
         $environment = new Environment();
         // This triggers the initialization
@@ -128,7 +138,7 @@ class EnvironmentTest extends TestCase
 
     public function testMergeConfigAfterInit()
     {
-        $this->expectException('RuntimeException');
+        $this->expectException(\RuntimeException::class);
 
         $environment = new Environment();
         // This triggers the initialization
@@ -146,11 +156,9 @@ class EnvironmentTest extends TestCase
         $this->assertContains($parser, $environment->getBlockParsers());
     }
 
-    /**
-     * @expectedException \RuntimeException
-     */
     public function testAddBlockParserFailsAfterInitialization()
     {
+        $this->expectException(\RuntimeException::class);
         $environment = new Environment();
 
         // This triggers the initialization
@@ -170,11 +178,9 @@ class EnvironmentTest extends TestCase
         $this->assertContains($renderer, $environment->getBlockRenderersForClass('MyClass'));
     }
 
-    /**
-     * @expectedException \RuntimeException
-     */
     public function testAddBlockRendererFailsAfterInitialization()
     {
+        $this->expectException(\RuntimeException::class);
         $environment = new Environment();
 
         // This triggers the initialization
@@ -199,11 +205,9 @@ class EnvironmentTest extends TestCase
         $this->assertEquals(1, preg_match($environment->getInlineParserCharacterRegex(), 'foo/bar'));
     }
 
-    /**
-     * @expectedException \RuntimeException
-     */
     public function testAddInlineParserFailsAfterInitialization()
     {
+        $this->expectException(\RuntimeException::class);
         $environment = new Environment();
 
         // This triggers the initialization
@@ -245,11 +249,9 @@ class EnvironmentTest extends TestCase
         $this->assertSame($processor, $environment->getDelimiterProcessors()->getDelimiterProcessor('*'));
     }
 
-    /**
-     * @expectedException \RuntimeException
-     */
     public function testAddDelimiterProcessorFailsAfterInitialization()
     {
+        $this->expectException(\RuntimeException::class);
         $environment = new Environment();
 
         // This triggers the initialization
@@ -269,11 +271,9 @@ class EnvironmentTest extends TestCase
         $this->assertContains($renderer, $environment->getInlineRenderersForClass('MyClass'));
     }
 
-    /**
-     * @expectedException \RuntimeException
-     */
     public function testAddInlineRendererFailsAfterInitialization()
     {
+        $this->expectException(\RuntimeException::class);
         $environment = new Environment();
 
         // This triggers the initialization
@@ -283,22 +283,46 @@ class EnvironmentTest extends TestCase
         $environment->addInlineRenderer('MyClass', $renderer);
     }
 
-    public function testGetBlockRendererForNonExistantClass()
+    public function testGetBlockRendererForUnknownClass()
     {
         $environment = new Environment();
+        $mockRenderer = $this->createMock(BlockRendererInterface::class);
+        $environment->addBlockRenderer(FakeBlock3::class, $mockRenderer);
 
-        $renderer = $environment->getBlockRenderersForClass('MyClass');
-
-        $this->assertEmpty($renderer);
+        $this->assertEmpty($environment->getBlockRenderersForClass(FakeBlock1::class));
     }
 
-    public function testGetInlineRendererForNonExistantClass()
+    public function testGetBlockRendererForSubClass()
     {
         $environment = new Environment();
+        $mockRenderer = $this->createMock(BlockRendererInterface::class);
+        $environment->addBlockRenderer(FakeBlock1::class, $mockRenderer);
 
-        $renderer = $environment->getInlineRenderersForClass('MyClass');
+        // Ensure the parent renderer is returned
+        $this->assertFirstResult($mockRenderer, $environment->getBlockRenderersForClass(FakeBlock3::class));
+        // Check again to ensure any cached result is also the same
+        $this->assertFirstResult($mockRenderer, $environment->getBlockRenderersForClass(FakeBlock3::class));
+    }
 
-        $this->assertEmpty($renderer);
+    public function testGetInlineRendererForNonUnknownClass()
+    {
+        $environment = new Environment();
+        $mockRenderer = $this->createMock(InlineRendererInterface::class);
+        $environment->addInlineRenderer(FakeInline3::class, $mockRenderer);
+
+        $this->assertEmpty($environment->getInlineRenderersForClass(FakeInline1::class));
+    }
+
+    public function testGetInlineRendererForSubClass()
+    {
+        $environment = new Environment();
+        $mockRenderer = $this->createMock(InlineRendererInterface::class);
+        $environment->addInlineRenderer(FakeInline1::class, $mockRenderer);
+
+        // Ensure the parent renderer is returned
+        $this->assertFirstResult($mockRenderer, $environment->getInlineRenderersForClass(FakeInline3::class));
+        // Check again to ensure any cached result is also the same
+        $this->assertFirstResult($mockRenderer, $environment->getInlineRenderersForClass(FakeInline3::class));
     }
 
     public function testAddExtensionAndGetter()
@@ -311,11 +335,9 @@ class EnvironmentTest extends TestCase
         $this->assertContains($extension, $environment->getExtensions());
     }
 
-    /**
-     * @expectedException \RuntimeException
-     */
     public function testAddExtensionFailsAfterInitialization()
     {
+        $this->expectException(\RuntimeException::class);
         $environment = new Environment();
 
         // This triggers the initialization
@@ -340,18 +362,134 @@ class EnvironmentTest extends TestCase
         $this->assertSame($test, $matches[0]);
     }
 
-    public function testInjectablesGetInjected()
+    public function testGetInlineParserCharacterRegexForAsciiCharacters(): void
     {
         $environment = new Environment();
 
-        $parser = $this->getMockBuilder([BlockParserInterface::class, EnvironmentAwareInterface::class, ConfigurationAwareInterface::class])->getMock();
-        $parser->expects($this->once())->method('setEnvironment')->with($environment);
-        $parser->expects($this->once())->method('setConfiguration');
+        $parser1 = $this->createMock(InlineParserInterface::class);
+        $parser1->method('getCharacters')->willReturn(['*']);
+        $environment->addInlineParser($parser1);
 
+        $parser2 = $this->createMock(InlineParserInterface::class);
+        $parser2->method('getCharacters')->willReturn(['[']);
+        $environment->addInlineParser($parser2);
+
+        // This triggers the initialization which builds the regex
+        $environment->getInlineParsersForCharacter('');
+
+        $regex = $environment->getInlineParserCharacterRegex();
+
+        $this->assertSame('/^[^\*\[]+/', $regex);
+    }
+
+    public function testGetInlineParserCharacterRegexForMultibyteCharacters(): void
+    {
+        $environment = new Environment();
+
+        $parser1 = $this->createMock(InlineParserInterface::class);
+        $parser1->method('getCharacters')->willReturn(['*']);
+        $environment->addInlineParser($parser1);
+
+        $parser2 = $this->createMock(InlineParserInterface::class);
+        $parser2->method('getCharacters')->willReturn(['★']);
+        $environment->addInlineParser($parser2);
+
+        // This triggers the initialization which builds the regex
+        $environment->getInlineParsersForCharacter('');
+
+        $regex = $environment->getInlineParserCharacterRegex();
+
+        $this->assertSame('/^[^\*★]+/u', $regex);
+    }
+
+    public function testInjectableBlockParsersGetInjected()
+    {
+        $environment = new Environment();
+
+        $parser = new FakeBlockParser();
         $environment->addBlockParser($parser);
 
         // Trigger initialization
         $environment->getBlockParsers();
+
+        $this->assertSame($environment, $parser->getEnvironment());
+        $this->assertNotNull($parser->getConfig());
+    }
+
+    public function testInjectableBlockRenderersGetInjected()
+    {
+        $environment = new Environment();
+
+        $renderer = new FakeBlockRenderer();
+        $environment->addBlockRenderer('', $renderer);
+
+        // Trigger initialization
+        $environment->getBlockParsers();
+
+        $this->assertSame($environment, $renderer->getEnvironment());
+        $this->assertNotNull($renderer->getConfig());
+    }
+
+    public function testInjectableInlineParsersGetInjected()
+    {
+        $environment = new Environment();
+
+        $parser = new FakeInlineParser();
+        $environment->addInlineParser($parser);
+
+        // Trigger initialization
+        $environment->getBlockParsers();
+
+        $this->assertSame($environment, $parser->getEnvironment());
+        $this->assertNotNull($parser->getConfig());
+    }
+
+    public function testInjectableInlineRenderersGetInjected()
+    {
+        $environment = new Environment();
+
+        $renderer = new FakeInlineRenderer();
+        $environment->addInlineRenderer('', $renderer);
+
+        // Trigger initialization
+        $environment->getBlockParsers();
+
+        $this->assertSame($environment, $renderer->getEnvironment());
+        $this->assertNotNull($renderer->getConfig());
+    }
+
+    public function testInjectableDelimiterProcessorsGetInjected()
+    {
+        $environment = new Environment();
+
+        $processor = new FakeDelimiterProcessor();
+        $environment->addDelimiterProcessor($processor);
+
+        // Trigger initialization
+        $environment->getBlockParsers();
+
+        $this->assertSame($environment, $processor->getEnvironment());
+        $this->assertNotNull($processor->getConfig());
+    }
+
+    public function testInjectableEventListenersGetInjected()
+    {
+        $environment = new Environment();
+
+        $listener1 = new FakeEventListener(function () { });
+        $listener2 = new FakeEventListenerInvokable(function () { });
+
+        $environment->addEventListener('', [$listener1, 'doStuff']);
+        $environment->addEventListener('', $listener2);
+
+        // Trigger initialization
+        $environment->getBlockParsers();
+
+        $this->assertSame($environment, $listener1->getEnvironment());
+        $this->assertSame($environment, $listener2->getEnvironment());
+
+        $this->assertNotNull($listener1->getConfiguration());
+        $this->assertNotNull($listener2->getConfiguration());
     }
 
     public function testBlockParserPrioritization()
@@ -456,7 +594,7 @@ class EnvironmentTest extends TestCase
             $actualOrder[] = 'c';
         }, 10);
 
-        $environment->addEventListener(FakeEvent::class, function (FakeEvent $e) use ($event, &$actualOrder) {
+        $environment->addEventListener(FakeEvent::class, function (FakeEvent $e) {
             $this->fail('Propogation should have been stopped before here');
         });
 
@@ -468,11 +606,9 @@ class EnvironmentTest extends TestCase
         $this->assertEquals('b', $actualOrder[2]);
     }
 
-    /**
-     * @expectedException \RuntimeException
-     */
     public function testAddEventListenerFailsAfterInitialization()
     {
+        $this->expectException(\RuntimeException::class);
         $environment = new Environment();
         $event = $this->createMock(AbstractEvent::class);
 
@@ -481,4 +617,15 @@ class EnvironmentTest extends TestCase
         $environment->addEventListener(AbstractEvent::class, function (AbstractEvent $e) {
         });
     }
+
+    private function assertFirstResult($expected, iterable $actual)
+    {
+        foreach ($actual as $a) {
+            $this->assertSame($expected, $a);
+
+            return;
+        }
+
+        $this->assertSame($expected, null);
+    }
 }
